
/*
 * erlang.c
 * 
 * Iterative computation of Erlang B and C functions.
 * 
 * Created by NJG: 15:06:09  06-27-96
 *
 *  $Id: erlang.c,v 1.3 2007/01/02 02:15:35 plh Exp $
 * 
 */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

//--------------------------------------------------------------------

main(int argc, char *argv[])
{

   double          erlangB;
   double          erlangC;
   double          traffic;
   double          eb;
   double          rho;
   double          nrt;
   double          ql;

   long            m;
   long            servers;

   /* inputs */

   if (argc < 3) {
      printf("Usage: %s servers traffic\n", argv[0]);
      printf("NOTE:Per server load = traffic/servers\n");
      exit(1);
   }

   /* initialize variables */

   servers = atol(*++argv);
   traffic = atof(*++argv);
   rho     = traffic / servers;
   erlangB = traffic / (1 + traffic);

   /* Jagerman's algorithm */

   for (m = 2; m <= servers; m++) {
      eb      = erlangB;
      erlangB = eb * traffic / (m + eb * traffic);
   }

   erlangC = erlangB / (1 - rho + rho * erlangB);
   nrt     = 1 + erlangC / (servers * (1 - rho));
   ql      = traffic * nrt;

   /* outputs */

   printf("  %ld Server System\n", servers);
   printf("  -----------------\n");
   printf("  Traffic intensity:  %5.5lf\n", traffic);
   printf("  Server utilization: %5.5lf\n", rho);
   printf("  \n");
   printf("  Erlang B  (loss  prob): %5.5lf\n", erlangB);
   printf("  Erlang C  (queue prob): %5.5lf\n", erlangC);
   printf("  M/M/m    Normalized RT: %5.5lf\n", nrt);
   printf("  M/M/m    No. in System: %5.5lf\n", ql);

} // main

//--------------------------------------------------------------------

